function df = fetch_series(varargin)
% Download time series from DBnomics. 
% 
% If not `None`, `dimensions` parameter must be a `dict` of dimensions (`list` of `str`), like so:
% `{"freq": ["A", "M"], "country": ["FR"]}`.
% 
% If not `None`, `series_code` must be a `str`. It can be a series code (one series), or a "mask" (many series):
% - remove a constraint on a dimension, for example `M..PCPIEC_WT`;
% - enumerate many values for a dimension, separated by a '+', for example `M.FR+DE.PCPIEC_WT`;
% - combine these possibilities many times in the same SDMX filter.
%         
% If the rightmost dimension value code is removed, then the final '.' can be removed too: `A.FR.` = `A.FR`.
% 
% If not `None`, `series_ids` parameter must be a non-empty `list` of series IDs.
% A series ID is a string formatted like `provider_code/dataset_code/series_code`.
% 
% If `max_nb_series` is `None`, a default value of 50 series will be used.
% 
% Return a cell array.
% 
% Examples:
% 
% - fetch one series:
% fetch_series('provider_code', "IMF", 'dataset_code', "CPI", 'series_code', "M.FR+DE.PCPIEC_IX+PCPIA_IX")
% fetch_series('provider_code', "IMF", 'dataset_code', "CPI", 'series_code', ".FR.PCPIEC_WT")
% fetch_series('provider_code', "IMF", 'dataset_code', "CPI", 'series_code', "M..PCPIEC_IX+PCPIA_IX")
% 
% - fetch all the series of a dataset:
% fetch_series('provider_code', "AMECO", 'dataset_code', "UVGD", 'max_nb_series', 500);
% 
% - fetch many series from different datasets:
% fetch_series('series_ids', ["AMECO/ZUTN/EA19.1.0.0.0.ZUTN", "AMECO/ZUTN/DNK.1.0.0.0.ZUTN", "IMF/CPI/A.AT.PCPIT_IX"])
% 
% - fetch many series from the same dataset, searching by dimension:
% fetch_series('provider_code', "AMECO", 'dataset_code', "ZUTN", dimensions={"geo": ["dnk"]})
%         

default_api_base_url = 'https://api.db.nomics.world/v22/'; 
default_editor_base_url = 'https://editor.nomics.world/api/v1/';

p = inputParser;
validStringInput = @(x) isstring(x) || ischar(x) || iscellstr(x);
p.addParameter('provider_code', '', validStringInput);
p.addParameter('dataset_code', '', validStringInput);
p.addParameter('series_code', '', validStringInput); 
p.addParameter('dimensions', @ischar); %if no dimensions specified, provider_code & dataset_code MUST BE GIVEN
p.addParameter('series_ids', '',validStringInput);
p.addParameter('max_nb_series', NaN, @isnumeric);
p.addParameter('api_base_url', default_api_base_url, validStringInput);
p.addParameter('dbnomics_filters', '', @ischar);
p.KeepUnmatched = false;
p.parse(varargin{:});

if strcmp(p.Results.api_base_url(end),'/') == 0
    p.Results.api_base_url = [p.Results.api_base_url '/'];
end

if isempty(p.Results.dataset_code)
    if iscell(p.Results.provider_code)
        p.Results.series_ids = p.Results.provider_code;
        p.Results.provider_code = '';
    elseif ~isempty(p.Results.provider_code)
        p.Results.series_ids = {p.Results.provider_code};
        p.Results.provider_code = '';
    end
end

series_base_url = [p.Results.api_base_url 'series'];

if isa(p.Results.dimensions, 'function_handle') && isempty(p.Results.series_code) && isempty(p.Results.series_ids)
    if isempty(p.Results.provider_code) || isempty(p.Results.dataset_code)
        error('When you don''t use dimensions, you must specifiy provider_code and dataset_code.');
    end
    api_link = sprintf('%s/%s/%s?observations=1', series_base_url, p.Results.provider_code, p.Results.dataset_code);
end

if ~isa(p.Results.dimensions, 'function_handle')
    if isempty(p.Results.provider_code) || isempty(p.Results.dataset_code)
        error('When you use dimensions, you must specifiy provider_code and dataset_code.');
    end
    api_link = sprintf('%s/%s/%s?observations=1&dimensions=%s', series_base_url, p.Results.provider_code, p.Results.dataset_code, p.Results.dimensions); %jsonencode(
end

if ~isempty(p.Results.series_code)
    if isempty(p.Results.provider_code) || isempty(p.Results.dataset_code)
        error('When you use series_code, you must specifiy provider_code and dataset_code.');
    end
    api_link = sprintf('%s/%s/%s/%s?observations=1', series_base_url, p.Results.provider_code, p.Results.dataset_code, p.Results.series_code);
end

if ~isempty(p.Results.series_ids)
    if ~isempty(p.Results.provider_code) || ~isempty(p.Results.dataset_code)
        error('When you use series_ids, you must not specifiy provider_code nor dataset_code.');
    end
    api_link = sprintf('%s?observations=1&series_ids=%s', series_base_url, strjoin(p.Results.series_ids,','));
end
df = fetch_series_by_api_link(api_link, p.Results.dbnomics_filters, p.Results.max_nb_series, default_editor_base_url);
end

