function [datasets_dimensions, series_dims_by_dataset_code, series_list] = iter_series_info(api_link, max_nb_series)
% Iterate through series.docs returned by API
% Returns structs of dataset(s) dimensions and series.
% The answer can have a key 'dataset_dimensions' if only one dataset is returned by API, or 'datasets_dimensions' if
% more than one dataset is returned.
% - datasets_dimensions or dataset_dimensions don't change between calls
% - series is the current series
% Example:
%     {
%        'datasets_dimensions': {
%            "AMECO/ZUTN": {
%                "code": "ZUTN",
%                "converted_at": "2019-05-08T02:51:04Z",
%                "dimensions_codes_order": ["freq", "unit", "geo" ...],
%                ...
%            },
%            "CEPII/CHELEM-TRADE-GTAP": {
%                "code": "CHELEM-TRADE-GTAP",
%                "converted_at": "2019-01-29T15:53:30Z",
%                "dimensions_codes_order": ["exporter", "importer", "secgroup", ...],
%                ...
%            },
%       'series':
%     }

default_max_nb_series = 50;
total_nb_series = 0;

datasets_dimensions = struct();
series_dims_by_dataset_code = struct();

while (true)
    response_json = fetch_series_page(api_link, total_nb_series);
    series_page = response_json.series;
    num_found = series_page.num_found;
    
    if isnan(max_nb_series) && num_found > default_max_nb_series
        error("DBnomics Web API found %i series matching your request, %i, but you did not pass any value for the 'max_nb_series' argument, so a default value of %i was used. Please give a higher value (at least max_nb_series=%i), and try again.", ...
               num_found, default_max_nb_series, num_found);
    end
    
    page_nb_series = length(series_page.docs);
    total_nb_series = total_nb_series+page_nb_series;
    
    if ~isnan(max_nb_series)
        if total_nb_series == max_nb_series
            break;
        elseif total_nb_series > max_nb_series
            nb_remaining_series = page_nb_series - (total_nb_series - max_nb_series);
            series_page.docs = series_page.docs(1:nb_remaining_series-1);
        end
    end
    
    series_list = cell(1,length(series_page.docs));
    
    for ii = 1:length(series_page.docs)
        try
            series = series_page.docs{ii};
        catch
            series = series_page.docs(ii);
        end
        series.provider_code = regexprep(series.provider_code,'[^a-zA-Z0-9]','_');
        series.dataset_code = regexprep(series.dataset_code,'[^a-zA-Z0-9]','_');
        complete_dataset_code = [series.provider_code '_' series.dataset_code];
        if numel(fieldnames(datasets_dimensions)) == 0
            assert(isfield(response_json, 'datasets') || isfield(response_json, 'dataset'));
            if isfield(response_json, 'datasets')
                datasets_dimensions = response_json.datasets;
            else
                datasets_dimensions.(complete_dataset_code) = response_json.dataset;
            end
        end
        
        series_list{ii} = series;
        % Store series dimensions information for future use
        series_code = regexprep(series.series_code,'[^a-zA-Z0-9]','_');
        series_dims_by_dataset_code.(complete_dataset_code).(series_code) = series.dimensions;
    end

    assert(total_nb_series <= num_found);
    if total_nb_series == num_found
        break;
    end
end
end
